# -*- coding: utf-8 -*-

"""A getter for InterPro."""

import re

import requests

from bioversions.utils import Getter, VersionType

__all__ = [
    "InterProGetter",
]

SOLVE_RE = re.compile(r"(\d)(st|nd|rd|th)")


class InterProGetter(Getter):
    """A getter for InterPro."""

    bioregistry_id = "interpro"
    name = "InterPro"
    homepage_fmt = "ftp://ftp.ebi.ac.uk/pub/databases/interpro/{version}/"
    date_fmt = "%d %B %Y"
    version_type = VersionType.semver_minor

    def get(self):
        """Get the latest InterPro version number."""
        with requests.Session() as session:
            res = session.get(
                "https://ftp.ebi.ac.uk/pub/databases/interpro/current_release/release_notes.txt"
            )
            for line in res.iter_lines():
                line = line.decode("utf8").strip()
                if line.startswith("Release") and not line.startswith("Release Notes"):
                    line = line[len("Release ") :]
                    version, rest = line.split(",", 1)
                    return dict(version=version, date=_process_line(rest))
        raise ValueError


def _process_line(s: str) -> str:
    return SOLVE_RE.sub(r"\1", s.strip())


if __name__ == "__main__":
    InterProGetter.print()



# -*- coding: utf-8 -*-

"""A getter for RGD."""

import requests

from bioversions.utils import Getter, VersionType

__all__ = [
    "RGDGetter",
]

URL = "https://download.rgd.mcw.edu/data_release/GENES.RAT.txt"


class RGDGetter(Getter):
    """A getter for RGD."""

    bioregistry_id = "rgd"
    name = "Rat Genome Database"
    date_fmt = "%Y-%m-%d"
    version_type = VersionType.date

    def get(self):
        """Get the latest RGD version number."""
        with requests.Session() as session:
            res = session.get(URL, stream=True)
            lines = res.iter_lines(decode_unicode=True)
            next(lines)
            next(lines)
            date = next(lines).split(" ")[2].replace("/", "-")
        return date


if __name__ == "__main__":
    RGDGetter.print()



# -*- coding: utf-8 -*-

"""Utilities for interacting with Twitter."""

from functools import lru_cache
from typing import Optional

import pystow
import tweepy


@lru_cache(maxsize=1)
def _get_api(
    consumer_key: Optional[str] = None,
    consumer_secret: Optional[str] = None,
    access_token: Optional[str] = None,
    access_token_secret: Optional[str] = None,
) -> Optional[tweepy.API]:
    consumer_key = pystow.get_config("bioversions", "consumer_key", passthrough=consumer_key)
    consumer_secret = pystow.get_config(
        "bioversions", "consumer_secret", passthrough=consumer_secret
    )
    access_token = pystow.get_config("bioversions", "access_token", passthrough=access_token)
    access_token_secret = pystow.get_config(
        "bioversions", "access_token_secret", passthrough=access_token_secret
    )

    if consumer_key and consumer_secret and access_token and access_token_secret:
        auth = tweepy.OAuthHandler(consumer_key, consumer_secret)
        auth.set_access_token(access_token, access_token_secret)

        # Create API object
        api = tweepy.API(auth)
        return api

    return None


def post(text: str):
    """Post the message to Twitter."""
    api = _get_api()
    if api is None:
        return None
    return api.update_status(text)


if __name__ == "__main__":
    post("Twitter test!")



# -*- coding: utf-8 -*-

"""A getter for NPASS."""

from bioversions.utils import Getter, VersionType, get_soup

__all__ = [
    "NPASSGetter",
]

URL = "http://bidd.group/NPASS/"


class NPASSGetter(Getter):
    """A getter for NPASS."""

    bioregistry_id = "npass"
    name = "NPASS"
    version_type = VersionType.semver_minor

    def get(self) -> str:
        """Get the latest NPASS version number."""
        soup = get_soup(URL)
        for li in soup.find(name="footer").find(name="ul").findAll(name="li"):
            if li.text.startswith("Version:"):
                return li.text[len("Version: ") :]
        raise ValueError(f"could not parse NPASS version from {URL}")


if __name__ == "__main__":
    NPASSGetter.print()



# -*- coding: utf-8 -*-

"""A getter for ExPASy."""

from datetime import datetime

import requests

from bioversions.utils import Getter, VersionType

__all__ = [
    "ExPASyGetter",
]

URL = "ftp://ftp.expasy.org/databases/enzyme/enzuser.txt"


class ExPASyGetter(Getter):
    """A getter for ExPASy."""

    bioregistry_id = "eccode"
    name = "ExPASy"
    # date_version_fmt = "%d-%b-%Y"
    version_type = VersionType.date

    def get(self) -> str:
        """Get the latest ExPASy version number."""
        s = requests.Session()
        res = s.get(URL, stream=True)
        li = res.iter_lines()
        next(li)
        next(li)
        r = next(li).decode("utf8").strip()[len("Release of ") :]
        return datetime.strptime(r, "%d-%b-%Y").strftime("%Y-%m-%d")


if __name__ == "__main__":
    ExPASyGetter.print()
